/**
*  @file		    hd3Struct.h
*  @brief		hd3 struct
*  @author:    hd3 tream
*  @date		   03/02/2015
*  @version     initialize version
*  Copyright:   Shanghai Baosight Software Co., Ltd.
*/
#ifndef HD3_INCLUDE_GUARD_HD3STRUCT_H
#define HD3_INCLUDE_GUARD_HD3STRUCT_H

#include "data_types.h"
#include "hd3Enum.h"
#include "hd3MaskDefine.h"

#ifndef HD3_DEFINE_GUARD_HD3HANDLE
#define HD3_DEFINE_GUARD_HD3HANDLE
typedef void* HD3HANDLE;
#endif

// HD3_SOCKET 
#ifndef WIN32
#    ifndef SOCKET
#    define SOCKET int
#    endif
#else 
#    ifndef SOCKET
#        if defined(_WIN64)
             typedef __int64 INT_PTR, *PINT_PTR;
             typedef unsigned __int64 UINT_PTR, *PUINT_PTR;
#       else
             typedef _W64 int INT_PTR, *PINT_PTR;
             typedef _W64 unsigned int UINT_PTR, *PUINT_PTR;
#       endif
     typedef UINT_PTR        SOCKET;
#   endif
#endif

// epsilon of compare double with 0
#define HD3_EPSILON (double)0.00001

// Tag㹲
#define HD3_COMM_TAG_PROP										\
	char	szTagName[HD3_LEN_TAG_NAME * HD3_BASE];					\
	bool	bArchiving;											\
	uint32	nChangeDate;										\
	char	szChanger[HD3_LEN_USER_NAME * HD3_BASE];					\
	float	fCompDev;											\
	int32	nCompMaxTime;										\
	enum HD3_COMP_TYPE	nCompType;											\
	uint32	nCreationDate;										\
	char	szCreator[HD3_LEN_USER_NAME * HD3_BASE];					\
	uint8 nRecAccess;											\
	char	szRecNormalGroup[HD3_LEN_USER_NAME * HD3_BASE];			\
	char	szRecOwnerGroup[HD3_LEN_USER_NAME * HD3_BASE];			\
	char	szDescriptor[HD3_LEN_TAG_DESC * HD3_BASE];				\
	char	szDigitalSet[HD3_LEN_DIGITAL_SET_NAME * HD3_BASE];		\
	char	szEngUnits[HD3_LEN_ENG_UNITS * HD3_BASE];					\
	int32	nExcMaxTime;										\
	float	fExcDev;											\
	uint32	nTagID;												\
	enum HD3_TAG_TYPE	nTagType;											\
	uint8	nTagAccess;											\
	char	szTagNormalGroup[HD3_LEN_USER_NAME * HD3_BASE];			\
	char	szTagOwnerGroup[HD3_LEN_USER_NAME * HD3_BASE];			\
	bool	bScan;												\
	float	fSpan;												\
	float	fMinValue;											\
	enum HD3_TAG_CLASS	nTagClass;                     \
	enum HD3_TAG_AVALIABLE nTagAvaliable;              \
	uint8	nParam1;											\
	uint8	nParam2;											\
	uint8	nParam3;											\
	uint8	nParam4;											\
	uint8	nParam5;                                            \
	enum HD3_TAG_OPC_STATE nOpcState;							\
	char  szIMVString[HD3_LEN_EXT_STRING * HD3_BASE];			\
	char  szExtString1[HD3_LEN_EXT_STRING * HD3_BASE];			\
	char  szExtString2[HD3_LEN_EXT_STRING * HD3_BASE]

// ͨ
#define HD3_PT_TAG_EXT_PROP										\
	HD3_M char	szInstrumentAddress[HD3_LEN_INSTRUMENT_ADDRESS * HD3_BASE];	\
	uint8	nInstrumentAddressType;								\
	char	szCollectorName[HD3_LEN_COLLECTOR_NAME * HD3_BASE];		\
	int32	nCollectorID;										\
	char	szDeviceName[HD3_LEN_DEVICE_NAME * HD3_BASE];				\
	HD3_M uint32	nDeviceID;											\
	char	szScanGroupName[HD3_LEN_SCANGROUP_NAME * HD3_BASE];		\
	HD3_M uint32	nScanGroupID;                                   \
	enum HD3_DATA_ORDER nDataOrder

// 
#define HD3_ALARM_TAG_EXT_PROP									\
	char szTest1[HD3_LEN_ALARM_TEST * HD3_BASE];						\
	char szTest2[HD3_LEN_ALARM_TEST * HD3_BASE];						\
	char szTest3[HD3_LEN_ALARM_TEST * HD3_BASE];						\
	char szTest4[HD3_LEN_ALARM_TEST * HD3_BASE];						\
	char szAction1[HD3_LEN_ALARM_ACTION * HD3_BASE];					\
	char szAction2[HD3_LEN_ALARM_ACTION * HD3_BASE];					\
	char szAction3[HD3_LEN_ALARM_ACTION * HD3_BASE];					\
	char szAction4[HD3_LEN_ALARM_ACTION * HD3_BASE];					\
	char szSourceTagName[HD3_LEN_TAG_NAME * HD3_BASE];				\
	uint32 nAlarmGroupID;										\
	enum HD3_AM_ACK_TYPE nAutoAck;											\
	int32 nTimeDeadBand;										\
	float fValueDeadBand

// ͳƵ
#define HD3_STATS_TAG_EXT_PROP								\
	char szSourceTagName[HD3_LEN_TAG_NAME * HD3_BASE];			\
	enum HD3_STATS_TYPE nStatsType;										\
	int32 nSampleStartDate;									\
	enum HD3_STATS_SAMPLE_MODE nSampleMode;										\
	HD3_M int32 nSampleInterval;									\
	int32 nSampleBeginOffset;									\
	int32 nSampleEndOffset;									\
	/*TT*/uint8 nSampleMonthBeginDay;								\
	/*TT*/uint8 nSampleMonthEndDay;								\
	bool bBeginTimeStamp;									\
	int32 nTimeStampOffset;									\
	int32 nScanPeriod;										\
	float fPctGood;											\
	char szFilterExpr[HD3_LEN_FILTER_EXPR * HD3_BASE];			\
	enum HD3_STATS_DROP_BAD_MODE nDropBadMode;									\
	double dCountParam1;									\
	double dCountParam2;                               \
	enum HD3_STATS_SAMPLE_TYPE nSampleType;										\
	int32 nInterpOffset;										\
	int32 nInterpPeriod;										\
	enum HD3_STATS_CONVERSION_TYPE nConversionType;										\
	double dConversion;										\
	enum HD3_STATS_FILTER_FAIL_MODE nFilterFailMode;		\
	int32 nCalcPeriod;										\
	bool bReset;\
	bool bUseBadValue

// 
#define HD3_CALC_TAG_EXT_PROP						\
	enum HD3_CALC_MODE nCalcMode;										\
	int32 nCalcPeriod;										\
	char szCalcExpr[HD3_LEN_CALC_EXPR * HD3_BASE];                  \
	bool bCalcHistory;                                       \
	int32 nCalcStartDate;                                    \
	int32 nCalcEndDate;                                      \
	bool bHaveEndTime;						\
	char szSourceTagName[HD3_LEN_TAG_NAME * HD3_BASE]

// SPC
#define HD3_SPC_RULE \
	char szUCLTagName[HD3_LEN_TAG_NAME * HD3_BASE];\
	char szCLTagName[HD3_LEN_TAG_NAME* HD3_BASE];\
	char szLCLTagName[HD3_LEN_TAG_NAME* HD3_BASE];\
	char szUSLTagName[HD3_LEN_TAG_NAME* HD3_BASE];\
	char szLSLTagName[HD3_LEN_TAG_NAME* HD3_BASE];\
	char szResetTagName[HD3_LEN_TAG_NAME* HD3_BASE];\
	float64 dUCLDefaultValue;\
	float64 dCLDefaultVaule;\
	float64 dLCLDefaultValue;\
	bool bRule1;\
	bool bRule2;\
	bool bRule3;\
	bool bRule4;\
	bool bRule5;\
	bool bRule6;\
	bool bRule7;\
	bool bRule8;\
	uint32 nRule1M;\
	uint32 nRule2M;\
	uint32 nRule3M;\
	uint32 nRule4M;\
	uint32 nRule5M;\
	uint32 nRule6M;\
	uint32 nRule7M;\
	uint32 nRule8M;\
	uint32 nRule5N;\
	uint32 nRule6N;\
	bool bJudgeContinuous

// SPC
#define HD3_SPC_TAG_EXT_PROP									\
	char szSourceTagName[HD3_LEN_TAG_NAME * HD3_BASE];				\
	uint32 nAlarmGroupID;										\
	enum HD3_SPC_CTRL_CHART_TYPE nCtrlChartType;											\
	enum HD3_SPC_TRIGGER_MODE	nTriggerMode;											\
	int32 nSampleSize;											\
	int32 nSampleInterval;											\
	bool bAutoAck;											\
	HD3_SPC_RULE

/* TagԽṹ */
HD3_M;
typedef struct {
	HD3_COMM_TAG_PROP;
}HD3CommTagProp;

// ͨ
HD3_M;
typedef struct {
	HD3_COMM_TAG_PROP;
	HD3_PT_TAG_EXT_PROP;
}HD3PtTagProp;


//ͬ
typedef struct {
	char	szTagName[HD3_LEN_TAG_NAME * HD3_BASE];
	uint32  sync;
	uint32	period;
	float	comp_dev;
	uint32  changeversion;
}HD3CloudTagProp;

//ͬ
typedef struct {
	char	szTagName[HD3_LEN_TAG_NAME * HD3_BASE];
	uint32  sync;
	uint32	period;
	float	comp_dev;
	uint32  changeversion;
	uint32 synchroTime;
	uint32 integerNumber;
	uint32 effectiveNumber;
	float coefficient;
	uint32 collectInterval;
	uint32 sendFrequency;
	char    szSendMode[HD3_LEN_TAG_NAME * HD3_BASE];
	char    szEquipment[HD3_LEN_TAG_NAME * HD3_BASE];
	char	szCloudEnName[HD3_LEN_TAG_NAME * HD3_BASE];
	char	szCloudChName[HD3_LEN_TAG_NAME * HD3_BASE];
	char	szReferenceEnName[HD3_LEN_TAG_NAME * HD3_BASE];
	char	szReferenceChName[HD3_LEN_TAG_NAME * HD3_BASE];
	char	szReferenceSourceIdentifier[HD3_LEN_TAG_NAME * HD3_BASE];
	char    szUnitNumber[HD3_LEN_TAG_NAME];
	char	szFactory[HD3_LEN_TAG_NAME * HD3_BASE];
	char	szArea[HD3_LEN_TAG_NAME * HD3_BASE];
}HD3BigDataTagProp;

// 
typedef struct {
	HD3_COMM_TAG_PROP;
	HD3_ALARM_TAG_EXT_PROP;
}HD3AlarmTagProp;

// SPC
typedef struct {
	HD3_COMM_TAG_PROP;
	HD3_SPC_TAG_EXT_PROP;
}HD3SpcTagProp;

// ͳƵ
typedef struct {
	HD3_COMM_TAG_PROP;
	HD3_STATS_TAG_EXT_PROP;
}HD3StatsTagProp;

// 
typedef struct {
	HD3_COMM_TAG_PROP;
	HD3_CALC_TAG_EXT_PROP;
}HD3CalcTagProp;

// ֤Ϣ
typedef struct {
	enum HD3_LICENSE nLicense;
	int32 nValidSec;
	uint32 nClientNum;	
}HD3LicenseInfo;

/** ʱ19701000:00:0120381193:14:06 */
typedef struct {
	int32 month;				/**< 1-12 */
	int32 year;					/**< four digit */
	int32 day;					/**< 1-31 */ 
	int32 hour;					/**< 0-23 */ 
	int32 minute;				/**< 0-59 */ 
	int32 second;				/**< 0-59*/ 
	int32 mSecond;				/**< 0-999*/
	int32 timezone;				/**< -12~12*/
}HD3TimeStamp;

// ʱ
typedef struct {
	int32 nSec;
	int16 nMsec;
}HD3Time;

// ʱ
typedef struct {
	HD3Time left;		// 
	HD3Time right;		// 
	bool bLeftClosed;	// 俪Ǳ
	bool bRightClosed;  // 俪Ǳ
}HD3TimeRegion;

// //һͻϢ
// typedef struct {
// 	HD3_M uint32 nClientIP;
// 	char szMac[32];
// 	int32 nSocketNum;
// 	HD3_SOCKET nSocket[HD3_CLIENT_SOCKET_MAX];
// }HD3ClientConnInfo;
// 
// typedef struct {
// 	HD3_SOCKET nSocket;
// 	int32 nConnectTime;
// 	int32 nPackageRecvNum;
// 	int32 nPackageRecvTime;
// 	char szUserName[HD3_LEN_USER_NAME * HD3_BASE];
// }HD3SocketNetInfo;

// 
typedef struct {
	int32 nSec;
	int16 nMsec;
	uint16 nQuality;
	enum HD3_TAG_TYPE nTagType;
	struct
	{
		int8 nInt8;
		int16 nInt16;
		int32 nInt32;
		float32 fFloat32;
		float64 fFloat64;
		int8 nDigital;
		struct
		{
			char* pBuf;
			uint16 nLenBuf;
		}strBlob;
	}value;
}HD3Record;

// ѯTagѯݲѯ¼ѯSPC¼ѯ
typedef struct {
	int32 nItemID;
	enum HD3_RELATION nRelation;
	char szValue[HD3_LEN_QUERY_ITEM_VALUE * HD3_BASE];
}HD3FilterItem;

// 
typedef struct {
	uint32 nAlarmTagID;						/**< ID*/
	HD3Time tvStartTime;						/**< ʼʱ*/
	HD3Time tvEndTime;						/**< ֹʱ*/
	HD3Time tvAckTime;						/**< ȷʱ*/
	HD3Time tvCommentTime;					/**< עʱ*/

	enum HD3_AM_REC_ACK_STATE nAcked;							/**< ȷ״̬*/
	enum HD3_AM_REC_RESTORE_STATE nRestored;						/**< ָ״̬*/
	enum HD3_AM_REC_PRIORITY nPriority;						/**< ȼ*/
	enum HD3_AM_ACK_TYPE nAckType;							/**< ȷ*/
	enum HD3_AM_REC_TYPE nAlarmType;						/**< */

	char szSrcValue[HD3_LEN_VAR_TYPE_VALUE * HD3_BASE];	/**< Դֵ*/
	char szTest[HD3_LEN_ALARM_TEST * HD3_BASE];			/**< Υ*/
	char szComment[HD3_LEN_ALARM_COMMENT * HD3_BASE];	/**< ע*/
	char szAcker[HD3_LEN_USER_NAME * HD3_BASE];			/**< ȷ*/
	char szCommentor[HD3_LEN_USER_NAME * HD3_BASE];		/**< ע*/
}HD3AlarmRecord;

// SPC
typedef struct {
	uint32 nSPCTagID;							/**< SPCID*/
	HD3Time tvStartTime;							/**< ʼʱ*/
	HD3Time tvEndTime;							/**< ֹʱ*/
	HD3Time tvAckTime;							/**< ȷʱ*/
	HD3Time tvCommentTime;						/**< עʱ*/
	enum HD3_AM_REC_ACK_STATE nAcked;								/**< ȷ״̬*/
	enum HD3_AM__REC_ACK_TYPE nAckType;							/**< ȷֶͣԶ*/
	enum HD3_AM_REC_RESTORE_STATE nRestored;							/**< ָ״̬*/
	int8 nValue;								/**< ֵ*/
	uint32 nRule1M;							/**< 1Mֵ*/
	uint32 nRule2M;							/**< 2Mֵ*/
	uint32 nRule3M;							/**< 3Mֵ*/
	uint32 nRule4M;							/**< 4Mֵ*/
	uint32 nRule5M;							/**< 5Mֵ*/
	uint32 nRule5N;							/**< 5Nֵ*/
	uint32 nRule6M;							/**< 6Mֵ*/
	uint32 nRule6N;							/**< 6Nֵ*/
	uint32 nRule7M;							/**< 7Mֵ*/
	uint32 nRule8M;							/**< 8Mֵ*/
	char szSrcValue[HD3_LEN_VAR_TYPE_VALUE * HD3_BASE];	/**< Դֵ*/
	char szComment[HD3_LEN_ALARM_COMMENT * HD3_BASE];	/**< ע*/
	char szAcker[HD3_LEN_USER_NAME * HD3_BASE];			/**< ȷ*/
	char szCommentor[HD3_LEN_USER_NAME * HD3_BASE];		/**< ע*/
}HD3SpcRecord;

// iHyperDB 
typedef struct {
	char szName[HD3_LEN_ALARM_GROUP_NAME * HD3_BASE];	/**<  */
	char szDesc[HD3_LEN_ALARM_GROUP_DESC * HD3_BASE];	/**<  */
	uint32 nGroupID;						/**< ID */
	uint32 nParentID;						/**< һһΪռ "" */
}HD3AlarmGroup;

// iHyperDB ʷѯ
typedef struct {
	HD3Time tvStart;
	HD3Time tvEnd;
	int64 nAckMask;  // HD3AmAckStateMask
	int64 nTypeMask;  // HD3AmTypeMask
	int64 nPriorMask;  // HD3AmPriorityMask
	int64 nRestoreMask;  // HD3AmRestoreStateMask
}HD3AlarmQuery;

// iHyperDB SPCѯ
typedef struct {
	HD3Time tvStart;
	HD3Time tvEnd;
	int64 nAckMask;  // HD3AmAckStateMask
	int64 nRestoreMask;  // HD3AmRestoreStateMask
}HD3SPCQuery;

// iHyperDB Ȩ޹: û
typedef struct {
	char szUserName[HD3_LEN_USER_NAME * HD3_BASE];
	char szPasswd[HD3_LEN_PASSWORD * HD3_BASE];
	char szUserDesc[HD3_LEN_USER_DESC * HD3_BASE];
}HD3SecUser;

// iHyperDB Ȩ޹: 
typedef struct {
	char szGroupName[HD3_LEN_GROUP_NAME * HD3_BASE];
	char szGroupDesc[HD3_LEN_GROUP_DESC * HD3_BASE];
}HD3SecGroup;

// iHyperDB Ȩ޹: Ȩ
typedef struct {
	char szName[HD3_LEN_AUTH_NAME * HD3_BASE];
	char szDesc[HD3_LEN_AUTH_DESC * HD3_BASE];
	int32 nAuth;
}HD3SecAuthItem;

// iHyperDB Ȩ޹: 
typedef struct {
	char szTrustName[HD3_LEN_TRUST_NAME * HD3_BASE];
	HD3_M char szStartIP[HD3_LEN_IP * HD3_BASE];
	HD3_M char szEndIP[HD3_LEN_IP * HD3_BASE];
	char szUserName[HD3_LEN_USER_NAME * HD3_BASE];
}HD3SecTrust;

// iHyperDB Ȩ޹: ȫݿ:Tag㡢û
typedef struct {
	char szOwnGroupName[HD3_LEN_GROUP_NAME * HD3_BASE];
	char szOperateGroupName[HD3_LEN_GROUP_NAME * HD3_BASE];
	int8 nSecurity;
}HD3SecItem;

// ɼ
typedef struct {
	uint32 nID;
	uint32 nStatusTagID;
	enum HD3_COLLECTOR_TYPE nType;
	char szName[HD3_LEN_COLLECTOR_NAME * HD3_BASE];
	char szDescription[HD3_LEN_COLLECTOR_DESC * HD3_BASE];
	char szExtParam[HD3_LEN_COLLECTOR_EXT_PARAM * HD3_BASE];

}HD3Collector;

// 豸
typedef struct {
	uint32 nID;							 /**<Device ID. */
	uint32 nCollectorID;				 /**<Collector ID. */
	uint16 nPort;
	int32 nTimeout;
	uint32 nStatusTagID;
	int16 nTryTime;
	bool bIsPrimary;
	char szName[HD3_LEN_DEVICE_NAME * HD3_BASE];
	char szDescription[HD3_LEN_DEVICE_DESC * HD3_BASE];
	char szAddr[HD3_LEN_DEVICE_ADDRESS * HD3_BASE];
	char szExtParam[HD3_LEN_DEVICE_EXT_PARAM * HD3_BASE];
}HD3Device;

// ɨ
typedef struct {
	uint32 nID;							/**<Group ID. */
	char szName[HD3_LEN_SCANGROUP_NAME * HD3_BASE];
	char szDescrption[HD3_LEN_SCANGROUP_DESC * HD3_BASE];
	int32 nPeriod;
	int32 nPhase;
	enum HD3_SCAN_TYPE  nScanType;					/**<advise mode or not */
	uint32 nDeviceID;					/**<Device ID. */
	int32 nDataAddr;
	int32 nDataLen;
	char szExtParam[HD3_LEN_SCANGROUP_EXT_PARAM * HD3_BASE];
}HD3ScanGroup;

// ֤Ϣ
typedef struct {
	uint32 nTagNumber;		//nMaxTagNum
	uint32 nClientNumber;	//nMaxConNum
	char szVCODE[HD3_LEN_LICENSE_OEM_VCODE * HD3_BASE];
	uint8 nSupportSPC;
	uint8 nSupportZT;
	char szTempAlian[2];	//4ֽڱ
}HD3LicInfo;

// ״̬
typedef struct {
	char szName[HD3_LEN_DIGITAL_SET_NAME * HD3_BASE];
}HD3DigitalSet;

// ״̬Ϣ
typedef struct {
	uint32 nStateID;
	char szStateName[HD3_LEN_DIGITAL_SET_STATE_NAME * HD3_BASE];
	int32 nStateValue;
}HD3DsStateInfo;

// ɼ״̬Ϣ
typedef struct {
	uint16 nQuality;
	int32 nTotalTagNum;
	int32 nActiveTagNum;
}HD3CtStatusInfo;

// TagϢ
typedef struct {
	uint32 nTagID;
	char szTagName[HD3_LEN_TAG_NAME * HD3_BASE];
	enum HD3_TAG_TYPE nTagType;
	enum HD3_TAG_CLASS nTagClass;
} HD3TagBasicInfo;

// ͳز
typedef struct {	
	enum HD3_STATS_TYPE nStatsType;			//ͳ
	int32 nTimeRegionNum;		//ͳʱСֵΪ1ֵΪ128
	HD3TimeRegion* pTimeRegions;//ͳʱ䣬ֶ֧ͳʱ
	float fPercentGood;			//ζ(GoodݸٷֱȴڻڸñʱΪĽǿŵ)0-100
	enum HD3_STATS_SAMPLE_TYPE nSampleType;			//ʽ
	char szFilterExpr[HD3_LEN_FILTER_EXPR * HD3_BASE];		//ݹ˱ʽ, ַʾ
	enum HD3_STATS_FILTER_FAIL_MODE nFilterFailMode;		//˱ʽִʧʱԲݵĴ˻ǲ
	double dExtParam1;			//ͼʱĲ, ֵ仯Ĳ仯ĵһ䲻ĵһΪ[dExtParam1, dExtParam2]
	double dExtParam2;			//ͼʱĲ仯ĵڶ䲻ĵڶ
	int32 nOffset;				//ģʽΪֵʱһֵΪstartTime+nOffset, λΪ
	int32 nPeriod;				//ģʽΪֵʱֵļΪnPeriod, λΪ
	bool bUseBadValue;			//badֵǷͳƣtrueʾbadֵͳ
}HD3StRelatedParam;

// ı
typedef struct {
	uint32	nChangeID;
	enum HD3_MODULE_TYPE nModuleType;
	uint32 	nModuleID;
	enum HD3_ALTERATION_ENTRY nEntry;  // nChangeCategory
	enum HD3_ALTERATION_TYPE nAlterationType;
	uint32	nSourceID;
	int32 	nVersion;
}HD3AlterationReport;  //HDChangeHis

// Tag
typedef struct {
	int64 nCommMask;  // Ե
	int64 nExtMask;
}HD3Mask;

// ѯ
typedef struct {
	int32 nSize;
	HD3FilterItem* pItem;  // array
}HD3FilterItemSet;

// ѯָҳTagĲ
typedef struct {
	HD3FilterItemSet itemSet;
	HD3Mask mask;
	uint32 nStartID;
	int32 nCapacity;
}HD3PageQueryTagParam;

// iHyperDBڵϢýڵ㡢ݽڵ㣩
typedef struct {
	uint32 nID;
	char szName[HD3_LEN_NODE_NAME * HD3_BASE];
	char szDesc[HD3_LEN_NODE_DESC * HD3_BASE];
	char szAddress[HD3_LEN_ADDRESS * HD3_BASE];
	char szMachineName[HD3_LEN_ADDRESS * HD3_BASE];
	uint16 nPort;
	char szBakAddress[HD3_LEN_ADDRESS * HD3_BASE];
	char szBakMachineName[HD3_LEN_ADDRESS * HD3_BASE];
	uint16 nBakPort;
	int32 nTagCapacity;
}HD3DataNode;

// ͳͨò
typedef struct {
	HD3TimeRegion timeRegion;
	float fPercentGood;
}HD3StFuncComm;

// APIӲ
typedef struct {
	char szAddress[HD3_LEN_ADDRESS * HD3_BASE];
	uint16 nPort;
	char szBakAddress[HD3_LEN_ADDRESS * HD3_BASE];
	uint16 nBakPort;
	int32 nTimeout;
}HD3Connection;

// ͻϢ
typedef struct {
	uint32 nClientIp;
	char szMac[32];
	int32 nSocketNum;
	SOCKET nSocket[HD3_API_CLIENT_MAX_SOCKET_NUM];
}HD3ClientConnInfo;

// SOCKETϢ
typedef struct {
	SOCKET nSocket;
	int32 nConnectTime;
	int32 nPackageRecvNum;
	int32 nPackageRecvTime;
	char szUserName[HD3_LEN_USER_NAME * HD3_BASE];
}HD3SocketNetInfo;

// ڵϢ
typedef struct {
	int32 nPort;			//˿ں
	int32 nArchFileSizeMB;	//鵵ļĴС
	int32 nArchFileNum;		//鵵ļ
	int32 nSeqFileSizeMB;	//лļĴС
	int32 nSeqCacheFlushPeriodSec; // лˢƵ
	bool bAutoDisconn;				//ǷԶϿݽ
	int32 nAutoDisconnPeriodSec;		//Զݽӵĳʱʱ
	bool bSysInfoCollect;				//ǷɼϵͳϢ
	int32 nSysInfoCollectPeriodSec;		//ϵͳϢɼ
	bool bDiscardFutureData;				//Ƿʱδ
	int32 nDiscardFutureDataTimeDelterSec;		//ʱʱƫ
	int64 nMachineTimeMs;
	char szHDVersion[HD3_LEN_HD_VERSION * HD3_BASE];
}HD3DataNodeCfg;

// ڵ״̬
typedef struct {
	char szHostIP[HD3_LEN_ADDRESS* HD3_BASE];			//IP
	char szBackIP[HD3_LEN_ADDRESS* HD3_BASE];			//IP
	enum HD3_RY_STATUS nHostStatus;							//״̬
	enum HD3_RY_STATUS nBackupStatus;							//״̬
}HD3DataNodeStatus;

typedef struct {
	uint32 nID;					//ack id
	uint32 nAlarmTagID;				// alarm tag id
	uint32 nServerID;				//serverid
	uint32 nOrginalTagID;			//alarm tag orginal tag id
	HD3Time alarmTime;				//alarm time
	HD3Time ackTime;				//ack time
	uint32 nAckerID;				//acker id
	char szAcker[HD3_LEN_USER_NAME * HD3_BASE];				//acker
}HD3Ack;

typedef struct {
	uint32 nID;					//comment id
	uint32 nAlarmTagID;				// alarm tag id
	uint32 nServerID;				//serverid
	uint32 nOrginalTagID;			//alarm tag orginal tag id
	HD3Time alarmTime;				//alarm time
	HD3Time commentTime;				//comment time
	uint32 nCommentorID;				//commentor id
	char szCommentor[HD3_LEN_USER_NAME * HD3_BASE];				//commentor
	char szComment[HD3_LEN_ALARM_COMMENT * HD3_BASE];		//comment
}HD3Comment;

// ýڵ״̬
typedef struct {
	char szHostIP[HD3_LEN_ADDRESS* HD3_BASE];			//IP
	char szBackIP[HD3_LEN_ADDRESS* HD3_BASE];			//IP
	enum HD3_RY_STATUS nHostStatus;							//״̬
	enum HD3_RY_STATUS nBackupStatus;							//״̬
}HD3ConfigCenterStatus;



#endif
